==> lsat-0.2.0.zip: lsat-0.2.0/lsatmain.c <==
/************************************************************************/
/* This is the main file for lsat					*/
/* It contains the main program and also does two checks of its own.    */
/* It gives the user some usage info (perhaps) and will check the 	*/
/* kernel version and release level of software running on the box.	*/
/*									*/
/* Current release is 0.1.8						*/
/* Project started on 04/01/2002					*/
/* homepage: http://www.dimlight.org/~number9/lsat 			*/
/* changelog: http://www.dimlight.org/~number9/lsat/changelog.html  	*/
/*									*/
/* This software Licensed under the GNU General Public License		*/
/* see the bottom of this lsatmain.c file for details.			*/
/*									*/
/* This program is _supposed_ to be modular in nature. That is to say:  */
/* all security checks are sub-functions called from this main prog.    */
/* As of now (release 0.1.1) the subfunctions can be called with 2	*/
/* parameters, the kernel level, and the release level. However, 	*/
/* the checkinetd.c program returns an integer dictating if inetd.conf  */
/* exists, and if it does another function hosts.c will check the 	*/
/* /etc/hosts.allow|deny files for existance, etc.			*/
/*									*/
/* I should not get in the habit of doing the above as it destroys the  */
/* modularity (sp?) of the program.					*/
/*									*/
/************************************************************************/	

/****************/
/* Includes	*/
/****************/
#include <stdio.h> /* fprintf,fgets,fputs,fopen,fclose,perror,sscanf */
#include <stdlib.h> /* the std stuff */
#include <string.h> /* strtok */
#include "lsatheader.h"

/********************************************************/
/* Usage function. Tell user command line options	*/
/********************************************************/
void usage()
{
	fprintf(stderr, "Usage is lsat [options]\n");
	fprintf(stderr, "Options:\n");
	fprintf(stderr, "		-v					verbose\n");
	fprintf(stderr, "		-o<filename>	Output file name\n");
	exit(8);
}

/***************************************************/
/* Check what version and kernel release we are at */
/* Changed: 4/4/2002:				   */
/* we will no longer cat /etc/redhat-release 	   */
/* in main... we will now just grab the 1st field  */
/* from the uname -a output so that it is easier   */
/* (later on too) to figure out what we should do. */
/* We shall still grab the "kernel/release" field  */
/* (field 3) to get the linux kernel or release    */
/* (e.g. Solaris version) that we are on. This (I  */
/* think) will give us greater flexibility in      */
/* porting to other *NIXs like Solaris or AIX.     */
/*						   */
/* Note: release will be field 1, kernel field 3   */
/***************************************************/
int versions(char release[], char kernel[], int verbose)
{
	/* some local vars... */
	FILE *infile;
	char *ptr;
	char line[1024];
	char tempstring[50];
	int  linecount=0;
	int  field=0;
	int  i=0;
	/* open input file for read... */
	infile = fopen("/tmp/templist.txt", "r");
	if (infile == NULL)
	{
		/* rhut-rho */
		perror("Can't open the file we just made!?!\n");
		perror("Giving up, check perms or something...\n");
		return(-1);
	}

	/* get a line from the file... it should contain only 2	*/
	/* 1st line is uname -a, so the third field is the kernel version */
	while (fgets(line, sizeof(line), infile) != NULL)
	{
		/* (re)set field...	*/
		field=0;
		ptr = strtok(line, ", \"\n");
		if (ptr == NULL)
		{
			break;
		}
		/* not blank?? Do some stuff...			*/
		while (ptr != NULL)
		{
			/* if linecount == 0, get 1st field */
			if (sscanf(ptr, "%s", &tempstring) < 1)
			{
				perror("sccanf bit the dust");
				return(-1);
			}
			if ((field == 0) && (linecount == 0))
			{
				for (i = 0; i < 50; i++)
				{
					release[i]=tempstring[i];
				}
			}
			if ((field == 2) && (linecount == 0))
			{
				for (i = 0; i < 50; i++)
				{
					kernel[i]=tempstring[i];
				}
			}
			ptr = strtok(NULL, ", \"\n");
			if (ptr == NULL)
			{
				/* freak out */
				break;
			}
			/* increment counter for the field we are reading */
			field++;
		} /* end while(ptr != NULL) */
		/* increment the line counter */
		linecount++;
	} /* end while (fgets...) */

	if (verbose == 1)
	{
		printf("System type is: %s\n", release);
		printf("Kernel/release level is: %s\n", kernel);
	}

	return(0);
}

/****************************************************************/
/* Main <smile> All sub-programs/functions should be called 	*/
/* from here. Note: they should also be listed in lsatheader.h  */
/****************************************************************/
int main(int argc, char *argv[])
{
	int retval=0;     /* a return value indicator		*/
	int verbose=0;    /* for verbosity			*/
	int inetexists=0; /* if inetd.conf exists, this is 1 	*/
	char release[50]; /* array for release level		*/
	char kernel[50];  /* what kernel user is running	*/
	char *out_file="lsat.out";  /* output filename if none given	*/
	FILE *tempptr;    /* a temp file pointer		*/

	/* some options that are not yet implimented :) */
	while ((argc > 1) && (argv[1][0]=='-'))
	{
		/* argv[1][1] is 1st option char */
		switch (argv[1][1])
		{
			case 'v':
				verbose = 1;
				break;

			/* outfile 					*/
			/* -o<name> is format 	*/

			case 'o':
				out_file = &argv[1][2];
				break;

			default:
				printf("Bad option %s\n", argv[1]);
				usage();
		}
		++argv;
		--argc;
	}

	if (verbose == 1)
	{
		printf("Starting LSAT...\n");
	}

	/* First off, get the output of uname -a			*/
	/* this is silly, I should know how to do this more easily 	*/

	if ( (system("uname -a >> /tmp/templist.txt")) < 0)
	{
		/* rhut-rho...something bad happened. 			*/
		perror("Problem executing pwd...\n");
		return(0);
	}

	/* read that in, check kernel version and "release"	*/
	if (verbose == 1)
	{
		printf("Getting system information...\n");
	}
	retval = versions(release, kernel, verbose);
	if (retval < 0)
	{
		/* something went wrong... 			*/
		perror("Problem in versions function.\n");
		return(0);
	}

	if ( (system("rm -f /tmp/templist.txt")) < 0)
	{
		/* rhut-rho...something bad happened.		*/
		perror("Problem rm'ing tempfile...\n");
		return(0);
	}

	/* secondly, check to see if lsat.out exists.   */
	/* if it does, call it lsat.old and rm lsat.out	*/

	tempptr = fopen(out_file, "r");
	if (tempptr == NULL)
	{
		/* great the outfile does not exist */
		/* do nothing... <smile>	    */
		;		
	}
	else
	{
		/* the outfile exists. lets not     */
		/* append to it, but back it up     */
		fclose(tempptr);
		if (verbose == 1)
		{
			printf("Outfile lsat.out exists...\n");
			printf("Making backup copy to lsat.old.\n");
		}
		if ((system("mv -f lsat.out lsat.old")) < 0)
		{
			/* rhut-rho... problems... */
			perror("Problem backing up lsat.out file.\n");
			perror("Check perms on the cwd please...\n");
			return(-1);
		}
	}

	/* run checkrpms */
	retval = 0;
	retval = checkrpms(release, kernel, out_file, verbose);
	if (retval < 0)
	{
		/* something went wrong...                      */
                perror("Problem in checkrpms module.\n");
		perror("You should have seen errors...\n");
                return(0);
	}

	/* run checkinetd */
	/* if hosts exists run checkhostsfilen */
	inetexists = checkinetd(release, kernel, out_file, verbose);
	/* if inetexists == 1, it does!				*/
	/* if inetexists == 0, it does not...			*/
	/* if inetexists == -1, we had some problems.		*/
	if (inetexists == 1)
	{
		checkhostsfiles(out_file, verbose);
	}
	if (inetexists == 0)
	{
		/* do nothing... */
		;
	}
	if (inetexists == -1)
	{
		/* there was a problem. User should have seen	*/
		/* messages from checkinetd program.		*/
		/* do nothing here and continue...		*/
		;
	}

	/* check for setuid/setgid files */
	retval = 0;
	retval = checkset(out_file, verbose);
	if (retval < 0)
        {
                /* something went wrong...                      */
                perror("Problem in checkset module.\n");
                perror("You should have seen errors...\n");
                return(0);
        }


	/* check for world writable files/directories */
	retval = 0;
	retval = checkwrite(out_file, verbose);
	if (retval < 0)
        {
                /* something went wrong...                      */
                perror("Problem in checkwrite module.\n");
                perror("You should have seen errors...\n");
                return(0);
        }

	/* check for .exrc and .forward files */
	retval = 0;
	retval = checkdotfiles(out_file, verbose);
	if (retval < 0)
	{
		/* something went wrong...                      */
                perror("Problem in checkdotfiles module.\n");
                perror("You should have seen errors...\n");
                return(0);
        }


	/* check /etc/passwd for uid 0 other than */
	/* root and users not needed on the sys.  */
	retval = 0;
	retval = checkpasswd(out_file, verbose);
	if (retval < 0)
        {
                /* something went wrong...                      */
                perror("Problem in checkpasswd module.\n");
                perror("You should have seen errors...\n");
                return(0);
        } 

	/* check to see if sticky bits are set */
	/* on /tmp & /var/tmp. I will add more */
	/* to this later on...			*/
	retval = 0;
	retval = checkfiles(out_file, verbose);
	if (retval < 0)
	{
		/* something went wrong...			*/
		perror("Problem in checkfiles module.\n");
		perror("You should have seen errors...\n");
		return(0);
	}


	/* by convention I will always have network */
	/* related checks at the bottom...	    */

	retval = 0;
	retval = checknet(out_file, verbose);
	if (retval < 0)
	{
		/* something went wrong...                      */
                perror("Problem in checknet module.\n");
                perror("You should have seen errors...\n");
		return(0);
        }


	/* tell the user we are finished and where the output is */
	printf("Finished.\n");
	printf("Check lsat.out for details.\n");
	return(0);
}


/**********************************************************************
Copyright (C) 2002 Robert Minvielle (number9)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
**********************************************************************/

==> lsat-0.3.2.zip: lsat-0.3.2/lsatmain.c <==
/************************************************************************/
/* This is the main file for lsat					*/
/* It contains the main program and also does two checks of its own.    */
/* It gives the user some usage info (perhaps) and will check the 	*/
/* kernel version and release level of software running on the box.	*/
/*									*/
/* Current release is 0.1.8						*/
/* Project started on 04/01/2002					*/
/* homepage: http://www.dimlight.org/~number9/lsat 			*/
/* changelog: http://www.dimlight.org/~number9/lsat/changelog.html  	*/
/*									*/
/* This software Licensed under the GNU General Public License		*/
/* see the bottom of this lsatmain.c file for details.			*/
/*									*/
/* This program is _supposed_ to be modular in nature. That is to say:  */
/* all security checks are sub-functions called from this main prog.    */
/* As of now (release 0.1.1) the subfunctions can be called with 2	*/
/* parameters, the kernel level, and the release level. However, 	*/
/* the checkinetd.c program returns an integer dictating if inetd.conf  */
/* exists, and if it does another function hosts.c will check the 	*/
/* /etc/hosts.allow|deny files for existance, etc.			*/
/*									*/
/* I should not get in the habit of doing the above as it destroys the  */
/* modularity (sp?) of the program.					*/
/*									*/
/************************************************************************/	

/****************/
/* Includes	*/
/****************/
#include <stdio.h> /* fprintf,fgets,fputs,fopen,fclose,perror,sscanf */
#include <stdlib.h> /* the std stuff */
#include <string.h> /* strtok */
#include "lsatheader.h"

/********************************************************/
/* Usage function. Tell user command line options	*/
/********************************************************/
void usage()
{
	fprintf(stderr, "Usage is lsat [options]\n");
	fprintf(stderr, "Options:\n");
	fprintf(stderr, "		-v					verbose\n");
	fprintf(stderr, "		-o<filename>	Output file name\n");
	exit(8);
}

/***************************************************/
/* Check what version and kernel release we are at */
/* Changed: 4/4/2002:				   */
/* we will no longer cat /etc/redhat-release 	   */
/* in main... we will now just grab the 1st field  */
/* from the uname -a output so that it is easier   */
/* (later on too) to figure out what we should do. */
/* We shall still grab the "kernel/release" field  */
/* (field 3) to get the linux kernel or release    */
/* (e.g. Solaris version) that we are on. This (I  */
/* think) will give us greater flexibility in      */
/* porting to other *NIXs like Solaris or AIX.     */
/*						   */
/* Note: release will be field 1, kernel field 3   */
/***************************************************/
int versions(char release[], char kernel[], int verbose)
{
	/* some local vars... */
	FILE *infile;
	char *ptr;
	char line[1024];
	char tempstring[50];
	int  linecount=0;
	int  field=0;
	int  i=0;
	/* open input file for read... */
	infile = fopen("/tmp/templist.txt", "r");
	if (infile == NULL)
	{
		/* rhut-rho */
		perror("Can't open the file we just made!?!\n");
		perror("Giving up, check perms or something...\n");
		return(-1);
	}

	/* get a line from the file... it should contain only 2	*/
	/* 1st line is uname -a, so the third field is the kernel version */
	while (fgets(line, sizeof(line), infile) != NULL)
	{
		/* (re)set field...	*/
		field=0;
		ptr = strtok(line, ", \"\n");
		if (ptr == NULL)
		{
			break;
		}
		/* not blank?? Do some stuff...			*/
		while (ptr != NULL)
		{
			/* if linecount == 0, get 1st field */
			if (sscanf(ptr, "%s", &tempstring) < 1)
			{
				perror("sccanf bit the dust");
				return(-1);
			}
			if ((field == 0) && (linecount == 0))
			{
				for (i = 0; i < 50; i++)
				{
					release[i]=tempstring[i];
				}
			}
			if ((field == 2) && (linecount == 0))
			{
				for (i = 0; i < 50; i++)
				{
					kernel[i]=tempstring[i];
				}
			}
			ptr = strtok(NULL, ", \"\n");
			if (ptr == NULL)
			{
				/* freak out */
				break;
			}
			/* increment counter for the field we are reading */
			field++;
		} /* end while(ptr != NULL) */
		/* increment the line counter */
		linecount++;
	} /* end while (fgets...) */

	if (verbose == 1)
	{
		printf("System type is: %s\n", release);
		printf("Kernel/release level is: %s\n", kernel);
	}

	return(0);
}

/****************************************************************/
/* Main <smile> All sub-programs/functions should be called 	*/
/* from here. Note: they should also be listed in lsatheader.h  */
/****************************************************************/
int main(int argc, char *argv[])
{
	int retval=0;     /* a return value indicator		*/
	int verbose=0;    /* for verbosity			*/
	int inetexists=0; /* if inetd.conf exists, this is 1 	*/
	int distribution; /* what distro are we on?		*/
	/* 1=redhat, 2=debian */
	char release[50]; /* array for release level		*/
	char kernel[50];  /* what kernel user is running	*/
	char *out_file="lsat.out";  /* output filename if none given	*/
	FILE *tempptr;    /* a temp file pointer		*/

	/* some options that are not yet implimented :) */
	while ((argc > 1) && (argv[1][0]=='-'))
	{
		/* argv[1][1] is 1st option char */
		switch (argv[1][1])
		{
			case 'v':
				verbose = 1;
				break;

			/* outfile 					*/
			/* -o<name> is format 	*/

			case 'o':
				out_file = &argv[1][2];
				break;

			default:
				printf("Bad option %s\n", argv[1]);
				usage();
		}
		++argv;
		--argc;
	}

	if (verbose == 1)
	{
		printf("Starting LSAT...\n");
	}

	/* First off, get the output of uname -a			*/
	/* this is silly, I should know how to do this more easily 	*/

	if ( (system("uname -a >> /tmp/templist.txt")) < 0)
	{
		/* rhut-rho...something bad happened. 			*/
		perror("Problem executing pwd...\n");
		return(0);
	}

	/* read that in, check kernel version and "release"	*/
	if (verbose == 1)
	{
		printf("Getting system information...\n");
	}
	retval = versions(release, kernel, verbose);
	if (retval < 0)
	{
		/* something went wrong... 			*/
		perror("Problem in versions function.\n");
		return(0);
	}

	if ( (system("rm -f /tmp/templist.txt")) < 0)
	{
		/* rhut-rho...something bad happened.		*/
		perror("Problem rm'ing tempfile...\n");
		return(0);
	}

	/* simple check to see if we are on RedHat or Debian */
	if ((system("cat /etc/redhat-release 1>/dev/null 2>/dev/null >/dev/null")) == 0)
	{
		/* we are on redhat */
		distribution = 1;
	}
	if ((system("cat /etc/debian_version 1>/dev/null 2>/dev/null >/dev/null")) == 0)
	{
		/* we are on debian */
		distribution = 2;
	}


	/* secondly, check to see if lsat.out exists.   */
	/* if it does, call it lsat.old and rm lsat.out	*/

	tempptr = fopen(out_file, "r");
	if (tempptr == NULL)
	{
		/* great the outfile does not exist */
		/* do nothing... <smile>	    */
		;		
	}
	else
	{
		/* the outfile exists. lets not     */
		/* append to it, but back it up     */
		fclose(tempptr);
		if (verbose == 1)
		{
			printf("Outfile lsat.out exists...\n");
			printf("Making backup copy to lsat.old.\n");
		}
		if ((system("mv -f lsat.out lsat.old")) < 0)
		{
			/* rhut-rho... problems... */
			perror("Problem backing up lsat.out file.\n");
			perror("Check perms on the cwd please...\n");
			return(-1);
		}
	}

	/* run checkpkgs */
	retval = 0;
	retval = checkpkgs(release, kernel, distribution, out_file, verbose);
	if (retval < 0)
	{
		/* something went wrong...                      */
                perror("Problem in checkpkgs module.\n");
		perror("You should have seen errors...\n");
                return(0);
	}

	/* run checkinetd */
	/* if hosts exists run checkhostsfilen */
	inetexists = checkinetd(release, kernel, out_file, verbose);
	/* if inetexists == 1, it does!				*/
	/* if inetexists == 0, it does not...			*/
	/* if inetexists == -1, we had some problems.		*/
	if (inetexists == 1)
	{
		checkhostsfiles(out_file, verbose);
	}
	if (inetexists == 0)
	{
		/* do nothing... */
		;
	}
	if (inetexists == -1)
	{
		/* there was a problem. User should have seen	*/
		/* messages from checkinetd program.		*/
		/* do nothing here and continue...		*/
		;
	}

	/* check for setuid/setgid files */
	retval = 0;
	retval = checkset(out_file, verbose);
	if (retval < 0)
        {
                /* something went wrong...                      */
                perror("Problem in checkset module.\n");
                perror("You should have seen errors...\n");
                return(0);
        }


	/* check for world writable files/directories */
	retval = 0;
	retval = checkwrite(out_file, verbose);
	if (retval < 0)
        {
                /* something went wrong...                      */
                perror("Problem in checkwrite module.\n");
                perror("You should have seen errors...\n");
                return(0);
        }

	/* check for .exrc and .forward files */
	retval = 0;
	retval = checkdotfiles(out_file, verbose);
	if (retval < 0)
	{
		/* something went wrong...                      */
                perror("Problem in checkdotfiles module.\n");
                perror("You should have seen errors...\n");
                return(0);
        }


	/* check /etc/passwd for uid 0 other than */
	/* root and users not needed on the sys.  */
	retval = 0;
	retval = checkpasswd(out_file, verbose);
	if (retval < 0)
        {
                /* something went wrong...                      */
                perror("Problem in checkpasswd module.\n");
                perror("You should have seen errors...\n");
                return(0);
        } 

	/* check to see if sticky bits are set */
	/* on /tmp & /var/tmp. I will add more */
	/* to this later on...			*/
	retval = 0;
	retval = checkfiles(out_file, verbose);
	if (retval < 0)
	{
		/* something went wrong...			*/
		perror("Problem in checkfiles module.\n");
		perror("You should have seen errors...\n");
		return(0);
	}

	retval = 0;
	retval = checkumask(out_file, verbose);
	if (retval < 0)
	{
		/* something went wrong...                      */
                perror("Problem in checknet module.\n");
                perror("You should have seen errors...\n");
                return(0);
        }


	/* by convention I will always have network */
	/* related checks at the bottom...	    */

	retval = 0;
	retval = checknet(out_file, verbose);
	if (retval < 0)
	{
		/* something went wrong...                      */
                perror("Problem in checknet module.\n");
                perror("You should have seen errors...\n");
		return(0);
        }


	/* tell the user we are finished and where the output is */
	printf("Finished.\n");
	printf("Check %s for details.\n", out_file);
	return(0);
}


/**********************************************************************
Copyright (C) 2002 Robert Minvielle (number9)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
**********************************************************************/
